local AddName, AddonTable = ...
local L = AddonTable.L

------------------------------
--		Options Equates		--
------------------------------
local Both, Enhanced, Base, Damage, DamageTaken, Percent, Rating, Live, Static = 3, 1, 2, 1, 2, 1, 2, 1, 2

----------------------------------
--		Text Return Formats		--
----------------------------------
local Double_Percent_Format = { "%.2f%%", "%.0f", "%.2f%% (%.0f)", }

------------------------------
--		Enhancements		--
------------------------------
-- Critical
function AddonTable.FunctionList.CritChance(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Percent_Rating
	local totalCrit, critRating = 0, 0
	local returnText, decimals
	local critChance = GetCritChance("player")
	local critSpellChance = GetSpellCritChance("player")
	local critRangedChance = GetRangedCritChance("player")
	local meleeRating, rangedRating, spellRating = GetCombatRating(CR_CRIT_MELEE), GetCombatRating(CR_CRIT_RANGED), GetCombatRating(CR_CRIT_SPELL)
	local ratingTable = {meleeRating, rangedRating, spellRating}
	totalCrit = max(critChance, critSpellChance, critRangedChance)
	local baseCrit = totalCrit
	critRating = max(meleeRating, rangedRating, spellRating)
	
	totalCrit = totalCrit + AddonTable.combustion + AddonTable.skyreach + AddonTable.betweenEyes + AddonTable.overflowEnergy
	
	if options.Display_Basic then totalCrit = baseCrit end
	
	if options.Decimals_Crit == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Crit == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Crit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	

	if AddonTable.Band(EB, Percent) then returnText = (decimals):format(totalCrit) end
	if AddonTable.Band(EB, Rating) then returnText = critRating end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(totalCrit) .. " (" .. critRating .. ")" end
	
	HUD:UpdateText(data, returnText)
end

-- Haste
function AddonTable.FunctionList.Haste(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Percent_Rating
	local statFormat = Double_Percent_Format[EB]	
	local haste = GetHaste()
	local hasteRating = GetCombatRating(CR_HASTE_MELEE)
	local returnText, decimals

	if options.Display_Basic then haste = haste end
	
	if options.Decimals_Haste == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Haste == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Haste == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Percent) then returnText = (decimals):format(haste) end
	if AddonTable.Band(EB, Rating) then returnText = hasteRating end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(haste) .. " (" .. hasteRating .. ")" end	
	
	HUD:UpdateText(data, returnText)
end

-- Mastery
function AddonTable.FunctionList.Mastery(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Percent_Rating
	local statFormat = Double_Percent_Format[EB]	
	local mastery = GetMasteryEffect()
	local masteryRating = GetCombatRating(CR_MASTERY)
	local returnText, decimals
	
	if options.Display_Basic then mastery = mastery end
	
	if options.Decimals_Mastery == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Mastery == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Mastery == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Percent) then returnText = (decimals):format(mastery) end
	if AddonTable.Band(EB, Rating) then returnText = masteryRating end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(mastery) .. " (" .. masteryRating .. ")" end		
	
	HUD:UpdateText(data, returnText)	
end

-- Versatility
function AddonTable.FunctionList.Versatility(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Damage_Taken
	local verDamage = GetVersatilityBonus(29) + GetCombatRatingBonus(29)
	local verMitigate = GetVersatilityBonus(31) + GetCombatRatingBonus(31)
	local versaRating = GetCombatRating(29)
	local returnText, decimals
	
	if options.Display_Rating then verDamage = GetCombatRating(29); verMitigate = GetCombatRating(31) end
	
	if options.Decimals_Versatility == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Versatility == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Versatility == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Damage) then returnText = (decimals):format(verDamage) end
	if AddonTable.Band(EB, DamageTaken) then returnText = (decimals):format(verMitigate) end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(verDamage) .. " (" .. (decimals):format(verMitigate) .. ")" end
	
	HUD:UpdateText(data, returnText)	
end

-- Avoidance
function AddonTable.FunctionList.Avoidance(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Percent_Rating
	local statFormat = Double_Percent_Format[EB]	
	local avoidance = GetAvoidance()
	local avoidRating = GetCombatRating(CR_AVOIDANCE)
	local returnText, decimals
	
	if options.Display_Basic then avoidance = avoidance end

	if options.Decimals_Avoidance == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Avoidance == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Avoidance == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Percent) then returnText = (decimals):format(avoidance) end
	if AddonTable.Band(EB, Rating) then returnText = avoidRating end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(avoidance) .. " (" .. avoidRating .. ")" end	
	
	HUD:UpdateText(data, returnText)
end

-- Leech
function AddonTable.FunctionList.Leech(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Percent_Rating
	local statFormat = Double_Percent_Format[EB]	
	local leech = GetLifesteal()
	local leechRating = GetCombatRating(CR_LIFESTEAL)
	local returnText, decimals
	
	if options.Display_Basic then leech = leech end
	
	if options.Decimals_Leech == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Leech == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Leech == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Percent) then returnText = (decimals):format(leech) end
	if AddonTable.Band(EB, Rating) then returnText = leechRating end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(leech) .. " (" .. leechRating .. ")" end		
	
	HUD:UpdateText(data, returnText)
end

-- Speed
function AddonTable.FunctionList.Speed(HUD, data, options, ...)

	local EB = options.Speed_Static
	local speedColor, staticColor = "", ""
	local vehicleSpeed = GetUnitSpeed("vehicle")
	local fullSpeed, runSpeed, flightSpeed, swimSpeed = GetUnitSpeed("player")
	local staticSpeed
	local returnText
	local flyBase, runBase = 0, 0
	
	if IsFlying() then staticSpeed = flightSpeed
	elseif IsSwimming() then staticSpeed = swimSpeed
	elseif vehicleSpeed > 0 then staticSpeed = vehicleSpeed
	elseif UnitOnTaxi("player") then staticSpeed = fullSpeed
	else staticSpeed = runSpeed end	
	
	if fullSpeed == 0 and vehicleSpeed > 0 then fullSpeed = vehicleSpeed end
	
	fullSpeed = fullSpeed / 0.07
	staticSpeed = staticSpeed / 0.07
	
	if fullSpeed == 0 or fullSpeed == 100 then speedColor = ""
	elseif fullSpeed < 100 then speedColor = "|cffC41E3A"
	elseif fullSpeed > 100 then speedColor = "|cff00f26d" end	
	
	if options.Decimals_Speed == 0 then decimals = "%.0f%%"
	elseif options.Decimals_Speed == 1 then decimals = "%.1f%%"
	elseif options.Decimals_Speed == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end		
	
	if AddonTable.Band(EB, Live) then returnStat = speedColor .. (decimals):format(fullSpeed) end
	if AddonTable.Band(EB, Static) then returnStat = (decimals):format(staticSpeed) end
	if AddonTable.Band(EB, Both) then returnStat = speedColor .. (decimals):format(fullSpeed) .. " |r(" .. (decimals):format(staticSpeed) .. ")" end
	
	HUD:UpdateText(data, returnStat)
end
------------------------------------------------
